import { __awaiter } from 'tslib';
import { InjectionToken, Injectable, Inject, EventEmitter, Component, ElementRef, ViewChild, Input, Output, Optional, Self, HostBinding, HostListener, NgModule } from '@angular/core';
import { NgControl, FormsModule } from '@angular/forms';
import { MatFormFieldControl, MatFormFieldModule } from '@angular/material/form-field';
import { Subject } from 'rxjs';
import { coerceBooleanProperty } from '@angular/cdk/coercion';
import { FocusMonitor } from '@angular/cdk/a11y';
import { CommonModule } from '@angular/common';

class Formatter {
}

// import { Position, Range } from 'monaco-editor';

const MONACO_CDN_BASE_URL = 'https://cdnjs.cloudflare.com/ajax/libs/monaco-editor/0.23.0/';
/**
 * DEFAULT_OPTIONS: monaco.editor.IEditorOptions & monaco.editor.IGlobalEditorOptions
 */
const DEFAULT_OPTIONS = {
    theme: 'vs',
    language: 'typescript',
    contextmenu: false,
    minimap: {
        enabled: false
    },
    autoIndent: false,
    formatOnPaste: false,
    formatOnType: false,
    renderIndentGuides: false,
    lineNumbers: 'off',
    glyphMargin: false,
    folding: false,
    lineDecorationsWidth: 0,
    lineNumbersMinChars: 0,
    scrollbar: {
        vertical: 'hidden',
        horizontal: 'hidden',
        useShadows: false,
        alwaysConsumeMouseWheel: false,
        handleMouseWheel: false
    },
    wordWrap: 'off',
    overviewRulerBorder: false,
    hideCursorInOverviewRuler: true,
    fontFamily: 'Roboto',
    fontSize: 16,
    fixedOverflowWidgets: true,
    scrollBeyondLastLine: false,
    renderLineHighlight: 'none'
    // renderValidationDecorations: 'off'
};

const NGX_EXPRESSIONS_CONFIG = new InjectionToken('NGX_EXPRESSIONS_CONFIG');
class NgxExpressionsConfig {
    constructor(args) {
        this.monacoBaseUrl = MONACO_CDN_BASE_URL;
        this.monacoDefaultOptions = DEFAULT_OPTIONS;
        this.log = null;
        if (args == null) {
            return;
        }
        if (args.monacoBaseUrl != null) {
            this.monacoBaseUrl = args.monacoBaseUrl;
        }
        if (args.monacoDefaultOptions != null) {
            this.monacoDefaultOptions = args.monacoDefaultOptions;
        }
        if (args.configure != null) {
            this.configure = args.configure;
        }
        if (args.log != null) {
            this.log = args.log;
        }
    }
}

const MONACO_PATH = 'min/vs';
const MONACO_EDITOR_PATH = 'min/vs/editor/editor.main';
class MonacoLoaderService {
    constructor(config) {
        this.config = config;
        this.isReady = false;
        if (this.config != null && this.config.monacoBaseUrl != null) {
            this.baseUrl = this.config.monacoBaseUrl;
        }
        else {
            this.baseUrl = MONACO_CDN_BASE_URL;
        }
    }
    createEditor(element, code, language = 'typescript') {
        return __awaiter(this, void 0, void 0, function* () {
            yield this.loadRequire();
            return window.monaco.editor.create(element, {
                language
            });
        });
    }
    loadRequire() {
        return __awaiter(this, void 0, void 0, function* () {
            if (window.require != null) {
                yield this.waitForNamespace((window) => window.monaco != null);
                return true;
            }
            this.addScript(`${this.baseUrl}min/vs/loader.js`);
            yield this.waitForNamespace((window) => window.require != null);
            window.require.config({ paths: { vs: `${this.baseUrl}${MONACO_PATH}` } });
            window.require([`${this.baseUrl}${MONACO_EDITOR_PATH}`], () => { });
            yield this.waitForNamespace((window) => window.monaco != null);
            window.MonacoEnvironment = {
                getWorkerUrl: () => proxy
            };
            const proxy = URL.createObjectURL(new Blob([
                `	self.MonacoEnvironment = {
		baseUrl: '${this.baseUrl}min/'
	};
	importScripts('${this.baseUrl}min/vs/base/worker/workerMain.js');`
            ], { type: 'text/javascript' }));
            if (this.config != null && this.config.configure != null) {
                yield this.config.configure(window.monaco);
            }
        });
    }
    waitForNamespace(check, timeout = 60000) {
        return __awaiter(this, void 0, void 0, function* () {
            yield new Promise((resolve, reject) => __awaiter(this, void 0, void 0, function* () {
                let wait = true;
                const timeoutTimer = setTimeout(() => {
                    console.error('Timeout waiting for monaco to become ready');
                    wait = false;
                    reject('Timeout');
                }, timeout);
                while (wait === true) {
                    if (check(window) === true) {
                        clearTimeout(timeoutTimer);
                        return resolve();
                    }
                    yield new Promise((wake) => {
                        setTimeout(() => {
                            wake();
                        }, 500);
                    });
                }
                resolve();
            }));
        });
    }
    addScript(scriptUrl) {
        const element = document.createElement('script');
        element.type = 'text/javascript';
        element.src = scriptUrl;
        element.addEventListener('load', (ev) => { });
        document.body.appendChild(element);
    }
}
MonacoLoaderService.decorators = [
    { type: Injectable }
];
MonacoLoaderService.ctorParameters = () => [
    { type: NgxExpressionsConfig, decorators: [{ type: Inject, args: [NGX_EXPRESSIONS_CONFIG,] }] }
];

class EditorComponent {
    constructor(elementRef, monacoLoader) {
        this.elementRef = elementRef;
        this.monacoLoader = monacoLoader;
        this.language = 'typescript';
        this.monacoOptions = DEFAULT_OPTIONS;
        this.editorReady = new EventEmitter();
        this.valueChanged = new EventEmitter();
        this.validationChange = new EventEmitter();
    }
    get id() {
        if (this._id == null) {
            this._id = `code-${EditorComponent._identity}`;
            EditorComponent._identity++;
        }
        return this._id;
    }
    ngAfterViewInit() {
        return __awaiter(this, void 0, void 0, function* () {
            this.editor = yield this.monacoLoader.createEditor(this.codeRef.nativeElement, '', this.language);
            this.editor.updateOptions(this.monacoOptions);
            yield this.init();
            this.editorReady.emit(this.editor);
        });
    }
    ngOnDestroy() {
        return __awaiter(this, void 0, void 0, function* () {
            if (this.editor != null) {
                this.editor.dispose();
                this.editor = undefined;
            }
        });
    }
    init() {
        return __awaiter(this, void 0, void 0, function* () {
            this.editor.onKeyUp((e) => {
                this.valueChanged.emit(this.editor.getModel().getValue());
            });
            this.editor.onDidChangeModelDecorations((e) => {
                setTimeout(() => {
                    const model = this.editor.getModel();
                    if (model === null || model.getModeId() !== 'typescript') {
                        return;
                    }
                    const owner = model.getModeId();
                    const markers = window.monaco.editor.getModelMarkers({ owner });
                    const errors = markers
                        .filter((i) => i.severity > 1 && i.resource.path === `/${this.id}`)
                        .map((i) => i)
                        .map((i) => i.message);
                }, 100);
                // const owner = model.getModeId();
                // const markers = this.editor.getModel()..getModelMarkers({ owner: 'typescript' });
                // for (const x of markers) {
                //     console.log('x', x);
                // }
                // this.validationChange.emit(markers.map((i) => i.message));
            });
        });
    }
}
EditorComponent._identity = 1000;
EditorComponent.decorators = [
    { type: Component, args: [{
                selector: 'expr-editor',
                template: "<div class=\"expr editor\" #codeRef [attr.id]=\"id\"></div>\n",
                styles: [".expr.editor{display:block;height:calc(100% + 100px);min-height:100px}.expr.editor ::ng-deep .monaco-editor,.expr.editor ::ng-deep .monaco-editor-background{background-color:transparent!important}.expr.editor ::ng-deep .decorationsOverviewRuler{display:none}"]
            },] }
];
EditorComponent.ctorParameters = () => [
    { type: ElementRef },
    { type: MonacoLoaderService }
];
EditorComponent.propDecorators = {
    codeRef: [{ type: ViewChild, args: ['codeRef',] }],
    language: [{ type: Input }],
    monacoOptions: [{ type: Input }],
    editorReady: [{ type: Output }],
    valueChanged: [{ type: Output }],
    validationChange: [{ type: Output }]
};

const INTERPOLATION_REGEX = /^(\((.*?)\):\s?\w*\s?=>\s?\{\s*return\s*\(`?\s*?)\n(.*?)\n(`?\);\s*\})\s*$/s;
const EXPRESSION_REGEX = /^(\((.*?)\):\s?\w*\s?=>\s?\{\s*return\s*\(?\s*?)\n(.*?)\n(\);\s*\})\s*$/s;
const EXTENDED_REGEX = /^(\((.*?)\):\s?\w*\s?=>\s?\{\s*\(?\s*?)\n(.*?)\n(\})\s*$/s;
const GENERIC_PARSER = (log, regex, model) => {
    const fullCode = model.getValue();
    if (!fullCode.match(regex)) {
        regex.lastIndex = 0;
        const failedGroups = regex.exec(fullCode);
        console.warn('Failed to parse code', { failedGroups, fullCode, regex });
        return null;
    }
    regex.lastIndex = 0;
    const groups = regex.exec(fullCode);
    if (groups == null) {
        return null;
    }
    const begin = groups[1] || '';
    const parameters = groups[2] || '';
    const targetCode = groups[3] || '';
    const end = groups[4] || '';
    let headerRange = null;
    let targetRange = null;
    let endRange = null;
    const beginMatch = model.findNextMatch(begin, new window.monaco.Position(1, 1), false, true, null, false);
    if (beginMatch != null && beginMatch.range != null) {
        headerRange = beginMatch.range;
    }
    else {
        throw new Error('Cannot match header');
    }
    const targetMatch = model.findNextMatch(targetCode, headerRange.getEndPosition(), false, true, null, false);
    if (targetCode != null && targetCode.length > 0) {
        if (targetMatch != null && targetMatch.range != null) {
            targetRange = targetMatch.range;
        }
        else {
            if (log != null) {
                log('Cannot match target', [targetCode, groups]);
            }
            throw new Error('Cannot match target');
        }
    }
    else {
        targetRange = new window.monaco.Range(3, 1, 3, 1);
    }
    const endMatch = model.findNextMatch(end, targetRange.getEndPosition(), false, true, null, false);
    if (endMatch != null && endMatch.range != null) {
        endRange = window.monaco.Range.fromPositions(new window.monaco.Position(targetRange.endLineNumber + 1, 1), endMatch.range.getEndPosition());
    }
    else {
        throw new Error('Cannot match end');
    }
    const startingPosition = targetRange.getStartPosition();
    const workingRange = window.monaco.Range.fromPositions(startingPosition, endRange.getStartPosition());
    return {
        fullCode,
        targetCode,
        fullLineCount: fullCode.split(/\r\n|\r|\n/).length,
        targetLineCount: targetCode.split(/\r\n|\r|\n/).length,
        headerRange,
        workingRange,
        footerRange: endRange,
        parameters,
        startingPosition
    };
};
const ɵ0 = (parameters, returnType, targetCode) => {
    if (targetCode == null || targetCode.length === 0) {
        targetCode = '';
    }
    if (returnType !== 'string') {
        returnType = 'string';
    }
    return `(${parameters}): ${returnType} => {
return (\`
${targetCode}
\`);
}
`;
}, ɵ1 = (parameters, returnType, targetCode) => {
    if (targetCode == null || targetCode.length === 0) {
        targetCode = '';
    }
    return `(${parameters}): ${returnType} => {
return (
${targetCode}
);
}
`;
}, ɵ2 = (parameters, returnType, targetCode) => {
    if (targetCode == null || targetCode.length === 0) {
        targetCode = '';
    }
    return `(${parameters}): ${returnType} => {
${targetCode}
}`;
};
const FORMAT_RULES = {
    typescript: {
        interpolation: {
            regex: INTERPOLATION_REGEX,
            getTemplate: ɵ0,
            parseTemplate: GENERIC_PARSER
        },
        expression: {
            regex: EXPRESSION_REGEX,
            getTemplate: ɵ1,
            parseTemplate: GENERIC_PARSER
        },
        extended: {
            regex: EXTENDED_REGEX,
            getTemplate: ɵ2,
            parseTemplate: GENERIC_PARSER
        }
    }
};

class FormatterService {
    constructor(config) {
        this.config = config;
        this.logger = null;
        if (this.config != null && this.config.log != null) {
            this.logger = this.config.log;
        }
    }
    parseModel(language, formatRuleName, model) {
        if (model == null) {
            return null;
        }
        if (this.previousFullCode === model.getValue()) {
            return this.previousParseResult;
        }
        this.previousFullCode = model.getValue();
        const rule = this.getRule(language, formatRuleName);
        this.previousParseResult = rule.parseTemplate(this.logger, rule.regex, model);
        if (this.config != null && this.config.log != null) {
            // tslint:disable-next-line:no-console
            this.config.log('NgxExpressions Parse Result', [this.previousParseResult]);
        }
        return this.previousParseResult;
    }
    getTemplate(language, formatRuleName, parameters, returnType, targetCode) {
        const rule = this.getRule(language, formatRuleName);
        if (rule == null) {
            throw new Error(`Invalid formatRuleName '${formatRuleName}'`);
        }
        rule.regex.lastIndex = 0;
        if (targetCode != null && targetCode.trim().length > 0 && targetCode.match(rule.regex)) {
            this.log('Provided code already matches template.', targetCode);
            return targetCode;
        }
        return rule.getTemplate(parameters, returnType, targetCode || '');
    }
    setHiddenRanges(editor, parseResult) {
        this.log('NgxExpressions setHiddenRanges', {
            editor,
            parseResult
        });
        if (parseResult == null) {
            return;
        }
        editor.setHiddenAreas([parseResult.headerRange, parseResult.footerRange]);
        editor.revealPositionNearTop(editor.getPosition(), window.monaco.editor.ScrollType.Immediate);
    }
    updatedCursorPosition(editor, parseResult, e) {
        this.log('NgxExpressions updatedCursorPosition', {
            editor,
            parseResult
        });
        if (parseResult == null) {
            return;
        }
        if (parseResult.workingRange == null) {
            editor.setPosition(parseResult.startingPosition);
            editor.setScrollPosition({
                scrollTop: 0,
                scrollLeft: 0
            });
            editor.revealPositionNearTop(new window.monaco.Position(1, 1));
        }
        else {
            if (!parseResult.workingRange.containsPosition(e.position)) {
                if (e.position.isBefore(parseResult.workingRange.getStartPosition())) {
                    editor.revealPositionNearTop(parseResult.workingRange.getStartPosition());
                }
                else {
                    const end = parseResult.workingRange.getEndPosition();
                    editor.setPosition(end);
                }
            }
            editor.revealPositionNearTop(editor.getPosition(), window.monaco.editor.ScrollType.Immediate);
        }
    }
    updatedCursorSelection(editor, parseResult, e) {
        this.log('NgxExpressions updatedCursorSelection', {
            editor,
            parseResult
        });
        if (e.source === 'api' || e.reason === window.monaco.editor.CursorChangeReason.NotSet) {
            return;
        }
        if (e.selection.isEmpty()) {
            return;
        }
        if (parseResult == null) {
            return;
        }
        if (parseResult.workingRange == null) {
            editor.setSelection(window.monaco.Selection.fromPositions(parseResult.headerRange.getEndPosition(), parseResult.headerRange.getEndPosition()));
            editor.revealPositionNearTop(editor.getPosition(), window.monaco.editor.ScrollType.Immediate);
            return;
        }
        let selection = e.selection;
        let update = false;
        if (selection.intersectRanges(parseResult.headerRange)) {
            update = true;
            selection = selection.setStartPosition(parseResult.headerRange.endLineNumber + 1, 1);
        }
        if (selection.intersectRanges(parseResult.footerRange)) {
            update = true;
            selection = selection.setEndPosition(parseResult.footerRange.startLineNumber - 1, editor.getModel().getLineLength(parseResult.footerRange.startLineNumber - 1) + 1);
        }
        if (update) {
            editor.setSelection(selection);
        }
        editor.revealPositionNearTop(editor.getPosition(), window.monaco.editor.ScrollType.Immediate);
    }
    updateScrollPosition(editor, parseResult) {
        editor.revealPositionNearTop(editor.getPosition(), window.monaco.editor.ScrollType.Immediate);
    }
    updateModelDecorations(editor, parseResult, editorId, e) {
        this.log('NgxExpressions updateModelDecorations', {
            editor,
            parseResult
        });
        if (parseResult == null) {
            return;
        }
        const model = editor.getModel();
        if (model === null || model.getModeId() !== 'typescript') {
            return [];
        }
        const owner = model.getModeId();
        const markers = window.monaco.editor.getModelMarkers({ owner });
        this.log('NgxExpressions updateModelDecorations markers', {
            markers
        });
        return markers
            .filter((i) => i.severity > 1 && i.resource.path === editor.getModel().uri.path)
            .map((i) => i.message);
    }
    isKeypressAllowed(editor, parseResult, e, autoExpand = false) {
        this.log('NgxExpressions isKeypressAllowed', {
            editor,
            parseResult
        });
        if (parseResult == null) {
            return;
        }
        if (e.code === 'Enter' && autoExpand === false) {
            return false;
        }
        else if (e.code.startsWith('F') && e.code.length === 2) {
            return false;
        }
        else if (e.code === 'Delete' && !this.isDeleteKeyAllowed(editor, parseResult)) {
            return false;
        }
        else if (e.code === 'Backspace' && !this.isBackspaceKeyAllowed(editor, parseResult)) {
            return false;
        }
        return true;
    }
    isDeleteKeyAllowed(editor, parseResult) {
        this.log('NgxExpressions isDeleteKeyAllowed', {
            editor,
            parseResult
        });
        if (parseResult == null) {
            return;
        }
        const position = editor.getPosition();
        return (position.lineNumber < parseResult.fullLineCount - 3 ||
            (position.lineNumber === parseResult.fullLineCount - 3 &&
                position.column <= editor.getModel().getLineLength(position.lineNumber)) ||
            !editor.getSelection().isEmpty());
    }
    isBackspaceKeyAllowed(editor, parseResult) {
        this.log('NgxExpressions isBackspaceKeyAllowed', {
            editor,
            parseResult
        });
        if (parseResult == null) {
            return;
        }
        const position = editor.getPosition();
        return new window.monaco.Position(3, 1).isBefore(position);
    }
    getRule(language, formatRuleName) {
        if (FORMAT_RULES[language] == null) {
            throw new Error(`Formatting rules for language '${language}' are not implemented.`);
        }
        const rule = FORMAT_RULES[language][formatRuleName];
        if (rule == null) {
            throw new Error(`Invalid formatter rule name ${formatRuleName}`);
        }
        return rule;
    }
    log(message, ...params) {
        if (this.config != null && this.config.log != null) {
            this.config.log(message, params);
        }
    }
}
FormatterService.decorators = [
    { type: Injectable }
];
FormatterService.ctorParameters = () => [
    { type: NgxExpressionsConfig, decorators: [{ type: Inject, args: [NGX_EXPRESSIONS_CONFIG,] }] }
];

const SINGLE_LINE_HEIGHT_PX = 22;
const MAX_VISIBLE_LINE_COUNT = 8;
class InputComponent {
    constructor(config, focusMonitor, elementRef, ngControl, formatter) {
        this.config = config;
        this.focusMonitor = focusMonitor;
        this.elementRef = elementRef;
        this.ngControl = ngControl;
        this.formatter = formatter;
        this.language = 'typescript';
        this.output = 'template';
        this.describedBy = '';
        this.id = `expr-input-${++InputComponent.nextId}`;
        this.autofilled = false;
        this.controlType = 'expression';
        this.stateChanges = new Subject();
        this._parameters = '';
        this._mode = 'interpolation';
        this._returnType = 'string';
        this._validations = [];
        this._busy = true;
        this._disabled = false;
        this._focused = false;
        this._placeholder = '';
        this._required = false;
        this._destroy = new Subject();
        this._autosize = false;
        if (ngControl) {
            // Set the value accessor directly (instead of providing
            // NG_VALUE_ACCESSOR) to avoid running into a circular import
            this.ngControl.valueAccessor = this;
            ngControl.valueAccessor = this;
        }
    }
    get busy() {
        return this._busy === true;
    }
    get returnType() {
        return this._returnType;
    }
    set returnType(value) {
        const targetCode = this.getTargetCode();
        this._returnType = value;
        this.writeValue(targetCode);
    }
    get disabled() {
        return this._disabled;
    }
    set disabled(value) {
        this._disabled = coerceBooleanProperty(value);
        this.stateChanges.next();
    }
    get placeholder() {
        return this._placeholder;
    }
    set placeholder(value) {
        this._placeholder = value;
        this.stateChanges.next();
    }
    get autosize() {
        if (this.mode === 'extended') {
            return true;
        }
        return this._autosize;
    }
    set autosize(value) {
        this._autosize = value;
        setTimeout(() => {
            this.updateLayout();
        }, 50);
    }
    get required() {
        return this._required;
    }
    set required(value) {
        this._required = coerceBooleanProperty(value);
        this.stateChanges.next();
    }
    get value() {
        const parsed = this.getParseResult();
        if (parsed == null) {
            return null;
        }
        if (this.output === 'template') {
            return parsed.targetCode;
        }
        else if (this.output === 'function') {
            return parsed.fullCode;
        }
        throw new Error(`Output mode '${this.output}' not implemented!`);
    }
    set value(value) {
        this.writeValue(value);
        this.stateChanges.next();
    }
    get shouldLabelFloat() {
        return this.focused || !this.empty || this.busy;
    }
    get validationErrors() {
        return this._validations;
    }
    get styles() {
        if (this.autosize === false) {
            return null;
        }
        const parsed = this.getParseResult();
        if (parsed != null && parsed.targetLineCount > 1) {
            const height = parsed.targetLineCount * SINGLE_LINE_HEIGHT_PX;
            return {
                height: `${height}px`
            };
        }
        return null;
    }
    get scrolling() {
        if (this.autosize === false) {
            return false;
        }
        const parsed = this.getParseResult();
        if (parsed == null) {
            return false;
        }
        return parsed.targetLineCount > MAX_VISIBLE_LINE_COUNT;
    }
    get empty() {
        return (this._value == null ||
            this._parseResult == null ||
            this._parseResult.targetCode == null ||
            this._parseResult.targetCode.trim().length === 0);
    }
    get errorState() {
        const parsed = this.getParseResult();
        if (this.required === true || (parsed != null && parsed.targetCode.trim().length > 0)) {
            if (this._validations == null) {
                return false;
            }
            return this._validations.length > 0;
        }
        return false;
    }
    get focused() {
        return this._focused;
    }
    set focused(value) {
        this._focused = value;
        if (this._focused === true) {
            this.updateLayout();
        }
        else {
            this.reloadParseResult();
        }
        this.stateChanges.next();
    }
    get parameters() {
        return this._parameters;
    }
    set parameters(value) {
        const targetCode = this.getTargetCode();
        this._parameters = value;
        this.writeValue(targetCode);
    }
    get mode() {
        return this._mode;
    }
    set mode(value) {
        const targetCode = this.getTargetCode();
        this._mode = value;
        this.writeValue(targetCode);
    }
    updateLayout() {
        this.updateHiddenRanges();
        if (this.editorRef == null || this.editorRef.editor == null) {
            return;
        }
        this.editorRef.editor.layout();
        this.editorRef.elementRef.nativeElement.scrollTop = 0;
    }
    ngAfterViewInit() {
        this.focusMonitor.monitor(this.elementRef.nativeElement, true).subscribe((focusOrigin) => {
            this.focused = !!focusOrigin;
        });
    }
    ngOnDestroy() {
        this._destroy.next();
        this._destroy.complete();
        this.stateChanges.complete();
        this.focusMonitor.stopMonitoring(this.elementRef.nativeElement);
    }
    onContainerClick(event) {
        if (event.target.tagName.toLowerCase() !== 'input') {
            this.focusMonitor.focusVia(this.editorRef.codeRef.nativeElement, 'mouse');
        }
    }
    onEditorReady() {
        return __awaiter(this, void 0, void 0, function* () {
            this.addListeners();
            this.applyValue();
            const parsed = this.getParseResult();
            if (parsed != null) {
                this.editorRef.editor.setPosition(parsed.startingPosition);
            }
            setTimeout(() => {
                this.updateLayout();
            }, 50);
            this._busy = false;
        });
    }
    registerOnChange(onChange) {
        this._onChange = onChange;
    }
    registerOnTouched(onTouched) {
        this._onTouched = onTouched;
    }
    setDescribedByIds(ids) {
        this.describedBy = ids.join(' ');
    }
    setDisabledState(shouldDisable) {
        this.editorRef.editor.updateOptions({ readOnly: shouldDisable });
        this.disabled = shouldDisable;
    }
    writeValue(value) {
        this._value = this.formatter.getTemplate(this.language, this.mode, this.parameters, this.returnType, value || '');
        this.applyValue();
    }
    onTouched() {
        if (this._onTouched != null) {
            this._onTouched();
        }
    }
    onChange(value) {
        if (this._onChange != null) {
            this._onChange(value || '');
        }
    }
    applyValue() {
        if (this.editorRef == null ||
            this.editorRef.editor == null ||
            this._value == null ||
            this._value.length === 0) {
            return;
        }
        let model = this.editorRef.editor.getModel();
        if (model == null) {
            model = window.monaco.editor.createModel(this._value, this.language, window.monaco.Uri.parse(`inmemory://${this.id}.ts`));
            this.editorRef.editor.setModel(model);
        }
        else {
            model.setValue(this._value);
        }
        this.formatter.setHiddenRanges(this.editorRef.editor, this.reloadParseResult());
    }
    addListeners() {
        if (this.editorRef == null || this.editorRef.editor == null) {
            throw new Error('Cannot add listeners as editor not ready.');
        }
        this.editorRef.editor.onDidChangeModelContent((e) => {
            this.formatter.setHiddenRanges(this.editorRef.editor, this.reloadParseResult());
            const parsed = this.getParseResult();
            if (parsed != null) {
                this.onChange(parsed.targetCode);
            }
        });
        this.editorRef.editor.onDidChangeCursorPosition((e) => {
            this.formatter.updatedCursorPosition(this.editorRef.editor, this.getParseResult(), e);
            this.onTouched();
        });
        this.editorRef.editor.onDidChangeCursorSelection((e) => {
            this.formatter.updatedCursorSelection(this.editorRef.editor, this.getParseResult(), e);
            this.onTouched();
        });
        this.editorRef.editor.onKeyDown((e) => {
            if (this.formatter.isKeypressAllowed(this.editorRef.editor, this.getParseResult(), e, this.autosize) ===
                false) {
                e.stopPropagation();
                e.preventDefault();
            }
            this.onTouched();
        });
        this.editorRef.editor.onDidScrollChange((e) => {
            this.formatter.updateScrollPosition(this.editorRef.editor, this.getParseResult());
        });
        this.editorRef.editor.onDidChangeModelDecorations((e) => {
            this._validations = this.formatter.updateModelDecorations(this.editorRef.editor, this.getParseResult(), this.id, e);
            if (this.validationErrors != null && this.validationErrors.length > 0) {
                this.onChange(this._value);
            }
        });
    }
    reloadParseResult() {
        if (this.editorRef == null || this.editorRef.editor == null) {
            return null;
        }
        const lineCount = this._parseResult != null ? this._parseResult.targetLineCount : -1;
        this._parseResult = this.formatter.parseModel(this.language, this.mode, this.editorRef.editor.getModel());
        if (this._parseResult != null && this._parseResult.targetLineCount !== lineCount) {
            this.updateLayout();
        }
        return this._parseResult;
    }
    getParseResult() {
        if (this._parseResult != null) {
            return this._parseResult;
        }
        return this.reloadParseResult();
    }
    updateHiddenRanges() {
        this.formatter.setHiddenRanges(this.editorRef.editor, this.getParseResult());
    }
    getTargetCode() {
        const parsed = this.reloadParseResult();
        if (parsed == null) {
            return '';
        }
        return parsed.targetCode;
    }
}
InputComponent.nextId = 0;
InputComponent.decorators = [
    { type: Component, args: [{
                providers: [{ provide: MatFormFieldControl, useExisting: InputComponent }],
                selector: 'expr-input',
                template: "<div\n    class=\"expr input\"\n    [class.busy]=\"busy === true\"\n    [class.disabled]=\"disabled === true\"\n    [class.primary]=\"color === 'primary'\"\n    [class.accent]=\"color === 'accent'\"\n    [class.autosize]=\"autosize === true\"\n    [class.scrolling]=\"scrolling === true\"\n    [ngClass]=\"mode\"\n    [ngStyle]=\"styles\"\n>\n    <expr-editor #editorRef [language]=\"language\" (editorReady)=\"onEditorReady()\" [ngStyle]=\"styles\"></expr-editor>\n    <div class=\"overlay\"></div>\n    <div class=\"spinner\"><i></i></div>\n</div>\n",
                styles: [".expr.input{position:relative;width:100%;min-height:22px;line-height:22px;height:22px;overflow:hidden;margin-bottom:-7px}.expr.input ::ng-deep .monaco-editor .scrollbar{display:none!important}.expr.input.autosize.scrolling .monaco-editor .scrollbar{display:inherit!important}.expr.input .overlay,.expr.input expr-editor{transition:opacity .3s ease;display:block}.expr.input expr-editor{opacity:1}.expr.input .overlay,.expr.input .spinner{display:none;opacity:0}.expr.input .overlay{position:absolute;top:0;bottom:0;left:0;right:0;z-index:9998}.expr.input .spinner{position:absolute;top:0;left:0;width:22px;height:22px}.expr.input .spinner i{position:absolute;top:0;left:4px;display:inline-block;width:16px;height:16px;-webkit-animation:circ-anim .7s linear infinite;animation:circ-anim .7s linear infinite;border-radius:50%;border-right:2px solid transparent;border-top:1px solid grey;transform-origin:center;z-index:9999}@-webkit-keyframes circ-anim{0%{transform:rotate(0)}to{transform:rotate(1turn)}}@keyframes circ-anim{0%{transform:rotate(0)}to{transform:rotate(1turn)}}.expr.input.busy expr-editor{opacity:0}.expr.input.busy .overlay,.expr.input.busy .spinner{display:block;opacity:1;z-index:99999}.expr.input.disabled expr-editor{opacity:.8}.expr.input.disabled .overlay{display:block;z-index:99999}.expr.input ::ng-deep .mtk1{color:#2d2d2d!important}.expr.input ::ng-deep .mtk20{color:#474747!important}"]
            },] }
];
InputComponent.ctorParameters = () => [
    { type: NgxExpressionsConfig, decorators: [{ type: Inject, args: [NGX_EXPRESSIONS_CONFIG,] }] },
    { type: FocusMonitor },
    { type: ElementRef },
    { type: NgControl, decorators: [{ type: Optional }, { type: Self }] },
    { type: FormatterService }
];
InputComponent.propDecorators = {
    returnType: [{ type: Input }],
    disabled: [{ type: Input }],
    placeholder: [{ type: Input }],
    autosize: [{ type: Input }],
    required: [{ type: Input }],
    value: [{ type: Input }],
    shouldLabelFloat: [{ type: HostBinding, args: ['class.floating',] }],
    editorRef: [{ type: ViewChild, args: ['editorRef',] }],
    color: [{ type: Input }],
    parameters: [{ type: Input }],
    mode: [{ type: Input }],
    output: [{ type: Input }],
    describedBy: [{ type: HostBinding, args: ['attr.aria-describedby',] }],
    id: [{ type: HostBinding }],
    updateLayout: [{ type: HostListener, args: ['window:resize', ['$event'],] }],
    onTouched: [{ type: HostListener, args: ['focusout',] }]
};

class NgxExpressionsModule {
    static forRoot(args) {
        return {
            ngModule: NgxExpressionsModule,
            providers: [
                MonacoLoaderService,
                FormatterService,
                {
                    provide: NGX_EXPRESSIONS_CONFIG,
                    useValue: args
                }
            ]
        };
    }
}
NgxExpressionsModule.decorators = [
    { type: NgModule, args: [{
                declarations: [EditorComponent, InputComponent],
                exports: [InputComponent],
                imports: [CommonModule, FormsModule, MatFormFieldModule]
            },] }
];

/**
 * Generated bundle index. Do not edit.
 */

export { InputComponent, NGX_EXPRESSIONS_CONFIG, NgxExpressionsConfig, NgxExpressionsModule, EditorComponent as ɵa, InputComponent as ɵb, FormatterService as ɵc, MonacoLoaderService as ɵd, FormatterService as ɵe, NGX_EXPRESSIONS_CONFIG as ɵf };
//# sourceMappingURL=ngx-expressions.js.map
